import React, { useState } from 'react';
import { Box, CssBaseline, ThemeProvider, createTheme, Tabs, Tab, TextField, IconButton } from '@mui/material';
import { Send as SendIcon, DeleteOutline as ClearIcon, Close as CloseIcon, Remove as MinimizeIcon, Fullscreen as MaximizeIcon } from '@mui/icons-material';
import { useChat } from './hooks/useChat';
import ChatPanel from './components/ChatPanel';
import FileExplorer from './components/FileExplorer';
import SettingsPanel from './components/SettingsPanel';
import HomePanel from './components/HomePanel';
import ScanningPanel from './components/ScanningPanel';
import ReportingPanel from './components/ReportingPanel';
import { ElectronProvider } from './contexts/ElectronContext';
import OllamaCheck from './components/OllamaCheck';

const theme = createTheme({
  palette: {
    mode: 'dark',
    primary: {
      main: '#2196f3', // Lighter blue
      light: '#64b5f6',
      dark: '#1976d2',
      contrastText: '#ffffff',
    },
  },
  components: {
    MuiCssBaseline: {
      styleOverrides: {
        body: {
          overflow: 'hidden',
        },
      },
    },
  },
});

interface TabPanelProps {
  children?: React.ReactNode;
  index: number;
  value: number;
}

function TabPanel(props: TabPanelProps & { [key: string]: any }) {
  const { children, value, index, ...other } = props;

  return (
    <Box
      role="tabpanel"
      hidden={value !== index}
      sx={{ 
        flex: value === index ? 1 : 'none',
        display: value === index ? 'flex' : 'none',
        flexDirection: 'column',
        overflow: 'auto',
        height: value === index ? '100%' : 0,
        minHeight: value === index ? 0 : 'none',
        '&::-webkit-scrollbar': {
          width: '8px',
          background: 'transparent',
        },
        '&::-webkit-scrollbar-thumb': {
          background: (theme) => theme.palette.divider,
          borderRadius: '4px',
          '&:hover': {
            background: (theme) => theme.palette.action.hover,
          },
        },
        '&::-webkit-scrollbar-track': {
          background: 'transparent',
        },
      }}
      {...other}
    >
      {value === index && (
        <Box sx={{ flex: 1, display: 'flex', flexDirection: 'column' }}>
          {children}
        </Box>
      )}
    </Box>
  );
}

function AppContent() {
  const [currentTab, setCurrentTab] = useState(0);
  const { isLoading, sendMessage, clearMessages, messages } = useChat();
  const [input, setInput] = useState('');

  const handleTabChange = (event: React.SyntheticEvent, newValue: number) => {
    setCurrentTab(newValue);
  };

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    if (!input.trim() || isLoading) return;

    const message = input;
    setInput('');
    setCurrentTab(1); // Switch to chat tab
    await sendMessage(message);
  };

  return (
    <ThemeProvider theme={theme}>
      <CssBaseline />
      <Box sx={{ 
        display: 'flex',
        flexDirection: 'column', 
        height: '100vh',
        overflow: 'hidden',
        bgcolor: 'background.default',
        color: 'text.primary',
      }}>
        {/* Custom titlebar */}
        <Box
          sx={{
            height: '28px',
            bgcolor: '#2f2f2f',
            display: 'flex',
            alignItems: 'center',
            WebkitAppRegion: 'drag',
            px: 1,
            position: 'relative',
            '& .window-controls': {
              WebkitAppRegion: 'no-drag',
              display: 'flex',
              gap: '8px',
              position: 'absolute',
              right: '8px',
              top: '50%',
              transform: 'translateY(-50%)',
            },
            '& .control-button': {
              width: '20px',
              height: '20px',
              display: 'flex',
              alignItems: 'center',
              justifyContent: 'center',
              border: 'none',
              cursor: 'pointer',
              transition: 'all 0.2s',
              bgcolor: '#1976d2',
              opacity: 0.7,
              '&:hover': {
                opacity: 1,
                bgcolor: '#2196f3',
              },
              '& svg': {
                width: '14px',
                height: '14px',
                color: '#fff',
              },
            },
          }}
        >
          <div className="window-controls">
            <button 
              className="control-button"
              onClick={() => window.electron.closeWindow()}
            >
              <CloseIcon />
            </button>
            <button 
              className="control-button"
              onClick={() => window.electron.minimizeWindow()}
            >
              <MinimizeIcon />
            </button>
            <button 
              className="control-button"
              onClick={() => window.electron.maximizeWindow()}
            >
              <MaximizeIcon />
            </button>
          </div>
        </Box>

        {/* Main content */}
        <Box sx={{ 
          display: 'flex',
          flex: 1,
          overflow: 'hidden',
        }}>
          <Box sx={{ 
            width: 300, 
            borderRight: '1px solid',
            borderColor: 'divider',
            display: 'flex',
            flexDirection: 'column',
            bgcolor: 'background.paper',
          }}>
            <FileExplorer />
          </Box>
          <Box sx={{ 
            flex: 1,
            display: 'flex',
            flexDirection: 'column',
            overflow: 'hidden',
            bgcolor: 'background.paper',
          }}>
            <Box sx={{ borderBottom: 1, borderColor: 'divider' }}>
              <Tabs value={currentTab} onChange={handleTabChange}>
                <Tab label="Home" />
                <Tab label="Chat" />
                <Tab label="Settings" />
                <Tab label="Scanning" />
                <Tab label="Reports" />
              </Tabs>
            </Box>
            <Box sx={{ 
              flex: 1, 
              overflow: 'hidden', 
              display: 'flex', 
              flexDirection: 'column',
              minHeight: 0,
              pb: '64px' // Add padding for the input height
            }}>
              <TabPanel value={currentTab} index={0}>
                <HomePanel />
              </TabPanel>
              <TabPanel value={currentTab} index={1}>
                <ChatPanel messages={messages} />
              </TabPanel>
              <TabPanel value={currentTab} index={2}>
                <SettingsPanel />
              </TabPanel>
              <TabPanel value={currentTab} index={3}>
                <ScanningPanel />
              </TabPanel>
              <TabPanel value={currentTab} index={4}>
                <ReportingPanel />
              </TabPanel>
            </Box>
          </Box>
        </Box>
        <Box
          component="form"
          onSubmit={handleSubmit}
          sx={{
            p: 2,
            borderTop: '1px solid',
            borderColor: 'divider',
            display: 'flex',
            gap: 1,
            bgcolor: 'background.paper',
            position: 'fixed',
            bottom: 0,
            right: 0,
            left: 0,
            zIndex: 1200,
            boxSizing: 'border-box'
          }}
        >
          <TextField
            fullWidth
            variant="outlined"
            placeholder="Ask a question..."
            value={input}
            onChange={(e) => setInput(e.target.value)}
            disabled={isLoading}
            size="small"
          />
          <IconButton 
            type="submit" 
            disabled={isLoading || !input.trim()}
            color="primary"
          >
            <SendIcon />
          </IconButton>
          <IconButton 
            onClick={clearMessages}
            color="error"
            disabled={messages.length === 0}
          >
            <ClearIcon />
          </IconButton>
        </Box>
      </Box>
    </ThemeProvider>
  );
}

function App() {
  const [ollamaInstalled, setOllamaInstalled] = useState(false);

  return (
    <ElectronProvider>
      {!ollamaInstalled ? (
        <OllamaCheck onInstalled={() => setOllamaInstalled(true)} />
      ) : (
        <AppContent />
      )}
    </ElectronProvider>
  );
}

export default App;
