import React, { useState, useEffect } from 'react';
import { Box, Button, Typography, CircularProgress, ThemeProvider, CssBaseline } from '@mui/material';
import { createTheme } from '@mui/material/styles';

const theme = createTheme({
  palette: {
    mode: 'dark',
    primary: {
      main: '#2196f3',
      light: '#64b5f6',
      dark: '#1976d2',
      contrastText: '#ffffff',
    },
    background: {
      default: '#121212',
      paper: '#1e1e1e',
    },
  },
});

interface OllamaCheckProps {
  onInstalled: () => void;
}

interface OllamaStatus {
  installed: boolean;
  running: boolean;
}

interface ModelStatus {
  installed: boolean;
  installing: boolean;
}

export function OllamaCheck({ onInstalled }: OllamaCheckProps) {
  const [ollamaStatus, setOllamaStatus] = useState<OllamaStatus | null>(null);
  const [modelStatus, setModelStatus] = useState<ModelStatus | null>(null);
  const [isChecking, setIsChecking] = useState(true);
  const [downloadProgress, setDownloadProgress] = useState<number>(0);
  const MODEL_NAME = 'damien113/datahound:latest';

  const checkOllama = async () => {
    try {
      const result = await window.electron.checkOllama();
      setOllamaStatus(result);
      if (result.installed && result.running) {
        checkModel();
      }
    } catch (error) {
      setOllamaStatus({ installed: false, running: false });
    } finally {
      setIsChecking(false);
    }
  };

  const checkModel = async () => {
    try {
      const status = await window.electron.checkModel(MODEL_NAME);
      setModelStatus(status);
      if (status.installed) {
        onInstalled();
      }
    } catch (error) {
      console.error('Error checking model:', error);
      setModelStatus({ installed: false, installing: false });
    }
  };

  const [installError, setInstallError] = useState<string | null>(null);

  const installModel = async () => {
    try {
      // Set initial installation state once at the start
      setModelStatus({ installed: false, installing: true });
      setInstallError(null);
      setDownloadProgress(0);
      let downloadComplete = false;
      
      await window.electron.pullModel(MODEL_NAME, (status) => {
        // Only update progress during download, avoid updating modelStatus
        const match = status.match(/downloading: ([\d.]+)%/);
        if (match) {
          const progress = parseFloat(match[1]);
          setDownloadProgress(progress);
        }
        
        if (status.includes('100% complete')) {
          downloadComplete = true;
          setDownloadProgress(100);
        }
      });

      // After pull completes, check the final status
      const status = await window.electron.checkModel(MODEL_NAME);
      if (status.installed) {
        // Show verification screen for a moment before transitioning
        await new Promise(resolve => setTimeout(resolve, 2000)); // 2 second delay
        setModelStatus({ installed: true, installing: false });
        onInstalled();
        return;
      }
      
      throw new Error(
        downloadComplete 
          ? 'Model download completed but installation verification failed. Please check Ollama logs.'
          : 'Model installation failed. Please try again.'
      );
    } catch (error) {
      console.error('Error installing model:', error);
      setModelStatus({ installed: false, installing: false });
      setInstallError(error instanceof Error ? error.message : 'Unknown error occurred');
      setDownloadProgress(0);
    }
  };

  const openOllamaWebsite = () => {
    window.electron.openExternal('https://ollama.com');
  };

  useEffect(() => {
    // Initial check
    checkOllama();

    // Only poll if Ollama is not installed or not running
    const interval = setInterval(() => {
      if (!ollamaStatus?.installed || !ollamaStatus?.running) {
        checkOllama();
      }
    }, 5000);

    // Cleanup interval on unmount
    return () => clearInterval(interval);
  }, [ollamaStatus?.installed, ollamaStatus?.running]);

  // Debounce progress updates
  const [debouncedProgress, setDebouncedProgress] = useState(0);
  useEffect(() => {
    const timeoutId = setTimeout(() => {
      setDebouncedProgress(downloadProgress);
    }, 100); // Debounce time of 100ms
    return () => clearTimeout(timeoutId);
  }, [downloadProgress]);

  if (isChecking) {
    return (
      <Box sx={{ 
        display: 'flex', 
        flexDirection: 'column', 
        alignItems: 'center', 
        justifyContent: 'center',
        height: '100vh',
        gap: 2,
        bgcolor: 'background.default',
        color: 'text.primary',
      }}>
        <CircularProgress sx={{ color: 'primary.main' }} />
        <Typography variant="h6" sx={{ color: 'text.primary' }}>
          Checking Ollama status...
        </Typography>
      </Box>
    );
  }

  if (!ollamaStatus?.installed) {
    return (
      <Box sx={{ 
        display: 'flex', 
        flexDirection: 'column', 
        alignItems: 'center', 
        justifyContent: 'center',
        height: '100vh',
        gap: 2,
        bgcolor: 'background.default',
        color: 'text.primary',
      }}>
        <Typography variant="h5" sx={{ color: 'text.primary', mb: 1 }}>
          Ollama is not installed
        </Typography>
        <Typography sx={{ color: 'primary.main', mb: 2, textAlign: 'center' }}>
          Ollama is required to use the AI features of this application.
        </Typography>
        <Button 
          variant="contained" 
          onClick={openOllamaWebsite}
          color="primary"
          size="large"
          sx={{
            px: 4,
            py: 1,
            borderRadius: 2,
            textTransform: 'none',
            fontSize: '1.1rem',
          }}
        >
          Install Ollama
        </Button>
      </Box>
    );
  }

  if (!ollamaStatus.running) {
    return (
      <Box sx={{ 
        display: 'flex', 
        flexDirection: 'column', 
        alignItems: 'center', 
        justifyContent: 'center',
        height: '100vh',
        gap: 2,
        bgcolor: 'background.default',
        color: 'text.primary',
      }}>
        <Typography variant="h5" sx={{ color: 'text.primary', mb: 1 }}>
          Starting Ollama
        </Typography>
        <Typography sx={{ color: 'primary.main', mb: 2, textAlign: 'center' }}>
          Attempting to start Ollama automatically...
          <br />
          This may take a few moments.
        </Typography>
        <CircularProgress size={24} sx={{ color: 'primary.main' }} />
      </Box>
    );
  }

  if (modelStatus === null || (!modelStatus.installed && !installError)) {
    return (
      <Box sx={{ 
        display: 'flex', 
        flexDirection: 'column', 
        alignItems: 'center', 
        justifyContent: 'center',
        height: '100vh',
        gap: 2,
        bgcolor: 'background.default',
        color: 'text.primary',
      }}>
        <Typography variant="h5" sx={{ color: 'text.primary', mb: 1 }}>
          {modelStatus?.installing ? 'Installing Data Hound AI Model' : 'AI Model Required'}
        </Typography>
        {installError && (
          <Typography sx={{ color: 'error.main', mb: 2, textAlign: 'center' }}>
            {installError}
          </Typography>
        )}
        <Typography sx={{ color: 'primary.main', mb: 2, textAlign: 'center' }}>
          {modelStatus?.installing ? (
            <>
              Downloading {MODEL_NAME}...
              <br />
              {debouncedProgress.toFixed(1)}% complete
              {downloadProgress === 100 && (
                <Typography component="div" sx={{ color: 'text.secondary', mt: 1 }}>
                  Verifying installation...
                </Typography>
              )}
            </>
          ) : (
            <>
              The required model {MODEL_NAME} is not installed.
              <br />
              Click below to install it.
            </>
          )}
        </Typography>
        {modelStatus?.installing ? (
          <CircularProgress 
            variant="determinate" 
            value={debouncedProgress}
            size={50}
            sx={{ color: 'primary.main' }}
          />
        ) : (
          <Button 
            variant="contained" 
            onClick={installModel}
            color="primary"
            size="large"
            sx={{
              px: 4,
              py: 1,
              borderRadius: 2,
              textTransform: 'none',
              fontSize: '1.1rem',
            }}
          >
            Download Data Hound AI Model
          </Button>
        )}
      </Box>
    );
  }

  return null;
}

export default function OllamaCheckWithTheme(props: OllamaCheckProps) {
  return (
    <ThemeProvider theme={theme}>
      <CssBaseline />
      <OllamaCheck {...props} />
    </ThemeProvider>
  );
}
