import { useState, useCallback, useEffect } from 'react';
import { useElectron } from './useElectron';
import type { DocumentMetadata } from '../../electron/types';

interface Message {
  role: 'user' | 'assistant';
  content: string;
  sources?: DocumentMetadata[];
}

export function useChat() {
  const electron = useElectron();
  const [messages, setMessages] = useState<Message[]>([]);
  const [isLoading, setIsLoading] = useState(false);

  useEffect(() => {
    let isStale = false;
    const handleChunk = (_: unknown, chunk: string) => {
      if (!isStale) {
        setMessages(prev => {
          const lastMessage = prev[prev.length - 1];
          if (lastMessage?.role === 'assistant') {
            // Update the last message's content with the new chunk
            const updatedMessages = [...prev];
            updatedMessages[prev.length - 1] = {
              ...lastMessage,
              content: lastMessage.content + chunk,
            };
            return updatedMessages;
          }
          return prev;
        });
      }
    };

    electron.on('llm-response-chunk', handleChunk);
    return () => {
      isStale = true;
      electron.off('llm-response-chunk', handleChunk);
    };
  }, [electron]);

  const sendMessage = useCallback(async (content: string) => {
    try {
      setIsLoading(true);
      setMessages(prev => [
        ...prev,
        { role: 'user', content },
        { role: 'assistant', content: '' }, // Add empty assistant message that will be updated with chunks
      ]);

      const response = await electron.queryLLM(content);
      
      // Final update with response content and sources
      setMessages(prev => {
        const updatedMessages = [...prev];
        const lastMessage = updatedMessages[updatedMessages.length - 1];
        if (lastMessage?.role === 'assistant') {
          lastMessage.content = lastMessage.content || response.answer;
          lastMessage.sources = response.sources;
        }
        return updatedMessages;
      });
    } catch (error) {
      console.error('Error sending message:', error);
      setMessages(prev => {
        const updatedMessages = [...prev];
        const lastMessage = updatedMessages[updatedMessages.length - 1];
        if (lastMessage?.role === 'assistant') {
          lastMessage.content = 'Sorry, there was an error processing your request.';
        }
        return updatedMessages;
      });
    } finally {
      setIsLoading(false);
    }
  }, [electron]);

  const clearMessages = useCallback(() => {
    setMessages([]);
  }, []);

  return {
    messages,
    isLoading,
    sendMessage,
    clearMessages,
  };
}
